@@ -6,67 +6,84 @@ DOCSTRING
 
 """
 
-import time
-import datetime
 
-import pyodbc
-
-import pandas as pd
 import backoff # this will help avoid Quote exceeded errors when making REST requests
-
+import datetime
 import httplib2
+import json
+import numpy as np
+import oauth2client
 import os
+import pandas as pd
+import pyodbc
 import sys
-
+import time
 from apiclient import discovery
 from apiclient.http import MediaFileUpload
-import oauth2client
+from googleapiclient.errors import HttpError
 from oauth2client import client
 from oauth2client import tools
-from oauth2client import file 
-from googleapiclient.errors import HttpError
+from oauth2client import file
 
 
 def read_connection_data_from_external_file(filepath, separator="="):
 	"""Reads SQL server connection information from an external file.
-	Keeping this information external is potentially important for security reasons.
+	Keeping this information external is potentially important for security
+	reasons.
+
+	Currently this method is capable of utilizing .txt and .json file types.
+	Other file types can be added as needed by the end users.
+
 	The format of this file should be:
 		server = [server_name]
 		database = [database_name]
 
 	Arguments:
-		filepath -- the location of the connection file (e.g. "C:/connection_data/server_connection_data.txt")
+		filepath -- the location of the connection file (e.g. "C:/
+			connection_data/server_connection_data.txt")
 		separator -- the delimiter (default "=")
 	Returns:
 		The server, database as strings
 	"""
-	with open(filepath, 'r') as f:
-		connection_data = f.readlines()
+	if filepath.endswith(".txt"):
+		with open(filepath, 'r') as f:
+			connection_data = f.readlines()
+			f.close()
 
-	connection_data_dict = dict()
+		connection_data_dict = dict()
 
-	# clean strings and split on delimiter
-	for entry in connection_data:
-		entry_cleaned = entry.replace(" ", "").strip("\n") # strip whitespace and trailing new lines
-		split_string = entry_cleaned.split(separator)
-		connection_data_dict[ split_string[0] ] = split_string[1]
-	
-	if "server" not in connection_data_dict or "database" not in connection_data_dict:
+		# clean strings and split on delimiter
+		for entry in connection_data:
+			# strip whitespace and trailing new lines
+			entry_cleaned = entry.replace(" ", "").strip("\n")
+			split_string = entry_cleaned.split(separator)
+			connection_data_dict[ split_string[0] ] = split_string[1]
+	elif filepath.endswith(".json"):
+		with open(filepath, "r") as f:
+			connection_data_dict = json.load(f)
+			f.close()
+	else:
+		raise ImportError("The Connection data file, specified by the " + \
+		"filepath parameter, is neither a .txt or .json file.")
+		exit(0)
+
+	if ("server" not in connection_data_dict) or \
+	("database" not in connection_data_dict):
 		raise ValueError(
-			"""Connection data file must contain server and database_name, formated like:
-			server = server_name
-			database = database_name\n""")
+			"Connection data file must contain server and database_name," + \"
+			"formated like: server = server_name database = database_name\n")
 		exit(0)
 
 	server = connection_data_dict["server"]
 	database = connection_data_dict["database"]
-	print("Server={}\nDatabase={}".format(server, database))
+	# print("Server={}\nDatabase={}".format(server, database))
 
 	return server, database
 
 def get_server_connection(server, database_name):
 	"""Tries to connect to the SQL database.
-	Attempts connection with driver SQL Server Native Client 10.0 and 11.0.
+	Attempts connection with driver SQL Server Native Client 10.0 and 11.0 and
+	a generic pyodbc sql server connection.
 
 	Arguments:
 		server -- the name of the server
@@ -76,24 +93,32 @@ def get_server_connection(server, database_name):
 	"""
 	cnn = False
 	try:
-		# make connection to GIS database
-		conn_info = 'DRIVER=SQL Server Native Client 10.0;' # ms sql server2008
-		conn_info = conn_info + 'SERVER={};'.format(server) # server name goes here
-		conn_info = conn_info + 'DATABASE_NAME={};'.format(database_name) # database name goes here
-		conn_info = conn_info + 'Trusted_Connection=yes' # use windows authentication
-		cnn = pyodbc.connect(conn_info)
-		print("Connected with SQL Server Native Client 10.0.")
+		# ms sql server2008
+		conn_info = 'DRIVER=SQL Server Native Client 10.0;'
+		conn_info += 'SERVER={};'.format(server)
+		conn_info += 'DABASE={};'.format(database_name)
+		conn_info += 'Trusted_Connection=yes'
+		# print("Connected with SQL Server Native Client 10.0.")
 	except pyodbc.Error as e1:
 		try:
-			conn_info = 'DRIVER=SQL Server Native Client 11.0;' # ms sql server2012
-			conn_info = conn_info + 'SERVER={};'.format(server) # server name goes here
-			conn_info = conn_info + 'DATABASE_NAME={};'.format(database_name) # database name goes here
-			conn_info = conn_info + 'Trusted_Connection=yes' # use windows authentication
+			 # ms sql server2012
+			conn_info = 'DRIVER=SQL Server Native Client 11.0;'
+			conn_info += 'SERVER={};'.format(server)
+			conn_info += 'DABASE={};'.format(database_name)
+			conn_info += 'Trusted_Connection=yes'
 			cnn = pyodbc.connect(conn_info)
-			print("Connected with SQL Server Native Client 11.0.")
+			# print("Connected with SQL Server Native Client 11.0.")
 		except pyodbc.Error as e2:
-			print("Error: {}".format(e2))
-
+			try:
+				# generic sql server driver
+				conn_info = "Driver={SQL Server};"
+				conn_info += "SERVER={};".format(server)
+				conn_info += "DABASE={};".format(database_name)
+				conn_info += "Trusted_Connection=yes"
+				cnn = pyodbc.connect(conn_info)
+				# print("Connected with a generic SQL Server Driver")
+			except pyodbc.Error as e3:
+				raise e3
 	return cnn
 
 def get_credentials(client_secret_file=None, application_name="SQL-google-interface", stored_credentials_dir="C:/credentials/"):
@@ -106,18 +131,22 @@ def get_credentials(client_secret_file=None, application_name="SQL-google-interf
 		client_secret_file -- location of the client secret file. (e.g. "C:/client_secret.json")
 			This only needs to be specified the first time the application is run.
 		application_name -- the name of the application (default "SQL-google-interface")
-		stored_credentials_dir -- the location where credentials will be stored (default "C:/credentials/") 
+		stored_credentials_dir -- the location where credentials will be stored (default "C:/credentials/")
 	Returns:
 		Credentials, the obtained credential.
 	"""
 
 	# If modifying these scopes, delete your previously saved credentials
-	scopes = 'https://www.googleapis.com/auth/drive https://www.googleapis.com/auth/spreadsheets'
+	scopes = [
+		'https://www.googleapis.com/auth/drive',
+		'https://www.googleapis.com/auth/spreadsheets']
 
 	if not os.path.exists(stored_credentials_dir):
 		os.makedirs(stored_credentials_dir)
-	credential_path = os.path.join(stored_credentials_dir, 'sql-drive-interface-credentials.json')
-	# print(credential_path)
+	credential_path = os.path.join(
+		stored_credentials_dir,
+		'sql-drive-interface-credentials.json')
+
 	store = oauth2client.file.Storage(credential_path)
 	credentials = store.get()
 	if not credentials or credentials.invalid:
@@ -155,7 +184,6 @@ def get_drive_service(credentials, service_type, version=None):
 
 	return discovery.build(service_type, version, http=http)
 
-
 def backoff_hdlr(details):
 	"""This is called when a function is backed off."""
 	print("\nBacking off {wait:0.1f} seconds afters {tries} tries "
@@ -186,20 +214,20 @@ def get_data_from_server(server_connection, SQL_filepath):
 	with open(SQL_filepath, 'r') as f:
 		query = f.read()
 
-	print("Retrieving data from SQL server... ")
+	# print("Retrieving data from SQL server... ")
 	dataframe = pd.read_sql(query, server_connection)
 
 	for col in dataframe:
 		if dataframe[col].dtype == "datetime64[ns]":
 			dataframe[col] = dataframe[col].apply(lambda x: x.strftime('%Y-%m-%d') if not pd.isnull(x) else '')
-	
+
 	# replace NaN and NaT with empty strings
 	dataframe.fillna('', inplace=True)
 
 	return dataframe
 
 def clean_dataframe(dataframe):
-	"""Type checks data in a pandas dataframe
+	"""Type checks data in a pandas dataframe using numpy's select method.
 
 	Arguments:
 		dataframe -- the dataframe to be cleaned
@@ -207,35 +235,31 @@ def clean_dataframe(dataframe):
 		A pandas dataframe containing the cleaned data
 	"""
 
-	data_as_list = dataframe.values.tolist()
-
-	# do lots of type checking -- there's probably a better (faster) way to do this, but it works for now
-	for row in data_as_list:
-		for idx in range(len(row)):
-			if type(row[idx]) is long or type(row[idx]) is float or type(row[idx]) is int:
-				row[idx] = str(row[idx])
-			elif row[idx] == None or row[idx] == "NULL":
-				row[idx] = ""
-			elif isinstance(row[idx], str):
-				# row[idx] is ordinary string
-				row[idx] = unicode(row[idx], "utf-8")
-			elif isinstance(row[idx], unicode):
-				pass
-			elif type(row[idx]) is pd.tslib.Timestamp:
-				row[idx] = row[idx].strftime('%d-%m-%Y')
-			elif type(row[idx]) is datetime.date:
-				row[idx] = row[idx].strftime('%d-%m-%Y')
-			else:
-				print("Type error")
-				row[idx] = "TYPE_ERROR"
-
-	# write data to spreadsheet
-	school_specific_data.insert(0, list(data.columns.values))
-	data_json = {'values': school_specific_data}
+	data = dataframe.copy().fillna("") # deal with the nulls first
 
 
-	return
+	for column in data.columns.to_list():
+	    con = [
+	        (data[column].dtype == np.object),
+	        (data[column].dtype == np.int64),
+	        (
+	            (data[column].dtype == "<M8[ns]") |
+	            (data[column].dtype == np.datetime64) |
+	            (data[column].dtype == datetime.datetime) |
+	            (data[column].dtype == datetime.date) |
+	            (data[column].dtype == pd.tslib.Timestamp)
+	        )
+	    ]
+	    choi = [
+	        data[column], # Python 3.x defaults to utf-8 so no need to force it
+	        data[column].astype(str), # Convert all int and float values to str
+			# convert any flavor of datetime object into a string containing
+			# only year, month, and day in the YYYY-mm-dd format
+	        data[column].dt.date.astype(str)
+	    ]
+	    data[column] = np.select(con, choi, "TYPE ERROR")
 
+	return data
 
 @backoff.on_exception(backoff.expo, HttpError, on_backoff=backoff_hdlr)
 def get_files_from_drive(drive_service, name=None, substring_name=None, mime_type=None, custom_metadata=None, parent_id=None, trashed=False, result_fields=["name", "id"]):
@@ -279,7 +303,7 @@ def get_files_from_drive(drive_service, name=None, substring_name=None, mime_typ
 		query_list.append("trashed = false")
 	elif trashed == True:
 		query_list.append("trashed = true")
-	
+
 
 	q = query_list[0]
 	for criterion in query_list[1:]:
@@ -294,12 +318,11 @@ def get_files_from_drive(drive_service, name=None, substring_name=None, mime_typ
 
 def create_permissions(drive_service, file_id):
 	"""
-
+	This function is not functional yet.
 	"""
 
 	pass
 
-
 @backoff.on_exception(backoff.expo, HttpError, on_backoff=backoff_hdlr)
 def create_spreadsheet(drive_service, spreadsheet_name, parent_folder_list=None, can_share='false', custom_metadata=None):
 	"""Creates a spreadsheet.
@@ -349,7 +372,7 @@ def delete_drive_files_by_ID(drive_service, list_of_file_ids):
 	batch = drive_service.new_batch_http_request(callback=batch_request_callback)
 	for fileID in list_of_file_ids:
 		#print('File ID: %s' % file.get('name'))
-		batch.add(drive_service.files().delete(fileId=fileID))	
+		batch.add(drive_service.files().delete(fileId=fileID))
 
 	batch.execute()
 
@@ -420,9 +443,9 @@ def format_spreadsheet(sheet_service, sheet_id, wrap_strategy=None):
 		})
 
 	batchUpdateRequest = {'requests': requests}
-	
+
 	sheet_service.spreadsheets().batchUpdate(spreadsheetId=sheet_id, body=batchUpdateRequest).execute()
-	
+
 	return sheet_id
 
 @backoff.on_exception(backoff.expo, HttpError, on_backoff=backoff_hdlr)
@@ -452,7 +475,7 @@ def populate_spreadsheet_from_df(sheets_service, sheet_id, dataframe):
 			else:
 				# print("Type error")
 				row[idx] = "TYPE_ERROR"
-	
+
 	# write data to spreadsheet
 	dataframe_cleaned.insert(0, list(dataframe.columns.values))
 	data_json = {'values': dataframe_cleaned}
@@ -462,7 +485,6 @@ def populate_spreadsheet_from_df(sheets_service, sheet_id, dataframe):
 	return sheet_id
 
 
-
 @backoff.on_exception(backoff.expo, HttpError, on_backoff=backoff_hdlr)
 def insert_file_into_folder(drive_service, folder_id, file_id):
 	"""Insert a file into a folder.
@@ -477,4 +499,4 @@ if __name__ == '__main__':
 		import argparse
 		flags = argparse.ArgumentParser(parents=[tools.argparser]).parse_args()
 	except ImportError:
-		flags = None
\ No newline at end of file
+		flags = None
